//
//  AppSettingViewController.m
//  LocationApp_Sample
//
//  Copyright (c) 2012 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "AppSettingViewController.h"

#import "SettingCell.h"

@interface AppSettingViewController ()

/* セクション定義 */
enum
{
    kSectionTop = 0,
    kSectionTokenServer,
    kSectionLocationServer,

    kAppSectionCount
};

/* 先頭セクションのセル定義 */
enum
{
    kSectionTopCellTransmitToServer = 0,
    kSectionTopCellLogUploadCount,
    kSectionTopCellDebugMode,
    kSectionTopCellDeleteLogFile,
    
    kSectionTopCellCount
};

/* "トークン発行サーバ"セクションのセル定義 */
enum
{
    kSectionTokenServerCellClientID = 0,
    kSectionTokenServerCellSecret,
    kSectionTokenServerCellUrl,

    kSectionTokenServerCellCount
};

/* "位置蓄積サーバ"セクションのセル定義 */
enum
{
    kSectionLocationServerCellApiKey = 0,
    kSectionLocationServerCellUrl,
    kSectionLocationServerCellDeviceID,
    kSectionLocationServerCellExtra,
    kSectionLocationServerCellUnsearchableExtra,
    kSectionLocationServerCellIsSearch,

    kSectionLocationServerCellCount
};

/* 設定値を更新 */
- (void)_textFieldValueChanged:(UITextField *)textfield;

/* 先頭セクションのセルを返す */
- (UITableViewCell *)_cellForTopSectionAtRow:(NSInteger)row;
/* "トークン発行サーバ"セクションのセルを返す */
- (UITableViewCell *)_cellForTokenServerSectionAtRow:(NSInteger)row;
/* "位置蓄積サーバ"セクションのセルを返す */
- (UITableViewCell *)_cellForLocationServerSectionAtRow:(NSInteger)row;

/* ログファイル削除の確認アラート表示 */
- (void)_showDeleteLogFileAlert;

/* ログファイルの削除 */
- (void)_deleteLogFileList;

@end


@implementation AppSettingViewController

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self != nil)
    {
    }
    return self;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    /* 設定値を更新 */
    [self.dataManager saveAppSettings];
}

#pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return kAppSectionCount;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [self _getSectionTitle:section];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch (section)
    {
        case kSectionTop:
        {
            return kSectionTopCellCount;
        }
        case  kSectionTokenServer:
        {
            return kSectionTokenServerCellCount;
        }
        case  kSectionLocationServer:
        {
            return kSectionLocationServerCellCount;
        }
        default:
            return 0;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    switch (indexPath.section)
    {
        case kSectionTop:
        {
            return [self _cellForTopSectionAtRow:indexPath.row];
        }
        case kSectionTokenServer:
        {
            return [self _cellForTokenServerSectionAtRow:indexPath.row];
        }
        case kSectionLocationServer:
        {
            return [self _cellForLocationServerSectionAtRow:indexPath.row];
        }
        default:
            return nil;
    }
}

#pragma mark UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    SettingCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    
    switch (indexPath.section)
    {
        case kSectionTop:
        {
            switch (indexPath.row)
            {
                case kSectionTopCellLogUploadCount:
                {
                    [self _showInputAlert:@"" cell:cell];
                    break;
                }
                case kSectionTopCellDeleteLogFile:
                {
                    [self _showDeleteLogFileAlert];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionTokenServer:
        {
            switch (indexPath.row)
            {
                case kSectionTokenServerCellClientID:
                case kSectionTokenServerCellSecret:
                case kSectionTokenServerCellUrl:
                {
                    [self _showInputAlert:@"トークン発行サーバ" cell:cell];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionLocationServer:
        {
            switch (indexPath.row)
            {
                case kSectionLocationServerCellApiKey:
                case kSectionLocationServerCellUrl:
                case kSectionLocationServerCellDeviceID:
                case kSectionLocationServerCellExtra:
                case kSectionLocationServerCellUnsearchableExtra:
                {
                    [self _showInputAlert:@"位置蓄積サーバ" cell:cell];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }
}

#pragma mark IBAction

/* 設定値リセット */
- (IBAction)resetSetting:(id)sender
{
    [self showResetAlert:^{
        [self.dataManager resetAppSettings];
        [self.tableView reloadData];
    }];
}

- (IBAction)switchValueChanged:(id)sender
{
    NSIndexPath *indexPath = [self indexPathForAccessoryView:sender];
    if (indexPath == nil)
    {
        return;
    }
    
    switch (indexPath.section)
    {
        case kSectionTop:
        {
            switch (indexPath.row)
            {
                case kSectionTopCellTransmitToServer:
                {
                    self.dataManager.permission = [sender isOn];
                    break;
                }
                case kSectionTopCellDebugMode:
                {
                    self.dataManager.debugMode = [sender isOn];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionLocationServer:
        {
            switch (indexPath.row)
            {
                case kSectionLocationServerCellIsSearch:
                {
                    self.dataManager.isSearch = [sender isOn];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }
}

#pragma mark Private

/* 設定値を更新 */
- (void)_textFieldValueChanged:(UITextField *)textfield
{
    NSIndexPath *indexPath = [self indexPathForAccessoryView:textfield];
    if (indexPath == nil)
    {
        return;
    }
    
    NSString *text = ( textfield.text.length == 0 )? nil: textfield.text;
    
    switch (indexPath.section)
    {
        case kSectionTop:
        {
            switch (indexPath.row)
            {
                case kSectionTopCellLogUploadCount:
                {
                    self.dataManager.logUploadCount = (text == nil)? IntEmptyValue: [text intValue];
                    textfield.text = [self textFromInt:self.dataManager.logUploadCount];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionTokenServer:
        {
            switch (indexPath.row)
            {
                case kSectionTokenServerCellClientID:
                {
                    self.dataManager.appTokenClientID = text;
                    break;
                }
                case kSectionTokenServerCellSecret:
                {
                    self.dataManager.appTokenSecret = text;
                    break;
                }
                case kSectionTokenServerCellUrl:
                {
                    self.dataManager.tokenUrl = text;
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionLocationServer:
        {
            switch (indexPath.row)
            {
                case kSectionLocationServerCellApiKey:
                {
                    self.dataManager.appApiKey = text;
                    break;
                }
                case kSectionLocationServerCellUrl:
                {
                    self.dataManager.locationUrl = text;
                    break;
                }
                case kSectionLocationServerCellDeviceID:
                {
                    self.dataManager.deviceID = text;
                    break;
                }
                case kSectionLocationServerCellExtra:
                {
                    self.dataManager.extra = text;
                    break;
                }
                case kSectionLocationServerCellUnsearchableExtra:
                {
                    self.dataManager.unsearchableExtra = text;
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }
}

/* 先頭セクションのセルを返す */
- (UITableViewCell *)_cellForTopSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionTop];
    
    switch (row)
    {
        case kSectionTopCellTransmitToServer:
        {
            return [self switchCell:title isOn:self.dataManager.permission];
        }
        case kSectionTopCellLogUploadCount:
        {
            return [self longTitleTextFieldCell:title text:[self textFromInt:self.dataManager.logUploadCount] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionTopCellDebugMode:
        {
            return[self switchCell:title isOn:self.dataManager.debugMode];
        }
        case kSectionTopCellDeleteLogFile:
        {
            return [self buttonCell:title];
        }
        default:
            return nil;
    }
}

/* "トークン発行サーバ"セクションのセルを返す */
- (UITableViewCell *)_cellForTokenServerSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionTokenServer];

    switch (row)
    {
        case kSectionTokenServerCellClientID:
        {
            return [self longTitleTextFieldCell:title text:self.dataManager.appTokenClientID keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionTokenServerCellSecret:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.appTokenSecret keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionTokenServerCellUrl:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.tokenUrl keyboardType:UIKeyboardTypeASCIICapable];
        }
        default:
            return nil;
    }
}

/* "位置蓄積サーバ"セクションのセルを返す */
- (UITableViewCell *)_cellForLocationServerSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionLocationServer];

    switch (row)
    {
        case kSectionLocationServerCellApiKey:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.appApiKey keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLocationServerCellUrl:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.locationUrl keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLocationServerCellDeviceID:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.deviceID keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLocationServerCellExtra:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.extra keyboardType:UIKeyboardTypeDefault];
        }
        case kSectionLocationServerCellUnsearchableExtra:
        {
            return [self longTitleTextFieldCell:title text:self.dataManager.unsearchableExtra keyboardType:UIKeyboardTypeDefault];
        }
        case kSectionLocationServerCellIsSearch:
        {
            return [self switchCell:title isOn:self.dataManager.isSearch];
        }
        default:
            return nil;
    }
}

/* アラート表示 */
- (void)_showInputAlert:(NSString *)title cell:(SettingCell *)cell
{
    [self showInputAlert:title cell:cell ok:^(NSString *value) {
        UITextField *_targetTextField = [cell textFieldInCell];
         _targetTextField.text = value;
         [self _textFieldValueChanged:_targetTextField];
    }];
}

/* ログファイル削除の確認アラート表示 */
- (void)_showDeleteLogFileAlert
{
    NSString *title = @"ログファイルを全て削除しますか？";
    NSString *message = nil;
    NSString *cancelButtonTitle = @"Cancel";
    NSString *okButtonTitle = @"OK";
    
    if (NSClassFromString(@"UIAlertController") != nil)
    {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title
                                                                                 message:message
                                                                          preferredStyle:UIAlertControllerStyleAlert];
        void (^handler)(UIAlertAction *action) = ^(UIAlertAction *action){
            if ([action.title isEqualToString:okButtonTitle])
            {
                [self _deleteLogFileList];
            }
        };
        [alertController addAction:[UIAlertAction actionWithTitle:cancelButtonTitle
                                                            style:UIAlertActionStyleCancel
                                                          handler:handler]];
        [alertController addAction:[UIAlertAction actionWithTitle:okButtonTitle
                                                            style:UIAlertActionStyleDefault
                                                          handler:handler]];
        
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

/* ログファイルの削除 */
- (void)_deleteLogFileList
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                         NSUserDomainMask,
                                                         YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];

    NSFileManager *manager = [NSFileManager defaultManager];
    NSError *error = nil;
    NSArray *fileList = [manager contentsOfDirectoryAtPath:documentsDirectory error:&error];
    if (fileList != nil)
    {
        for (NSString *name in fileList)
        {
            if (([name hasPrefix:@"LocationSample_"] == YES) &&
                ([[name pathExtension] isEqualToString:@"txt"] == YES))
            {
                NSString *removePath = [documentsDirectory stringByAppendingPathComponent:name];
                [manager removeItemAtPath:removePath error:&error];
            }
        }
    }
}

/* セクションのタイトルを返す */
- (NSString *)_getSectionTitle:(NSInteger)section
{
    switch (section)
    {
        case kSectionTop:
        {
            return @"";
        }
        case kSectionTokenServer:
        {
            return @"トークン発行サーバ";
        }
        case kSectionLocationServer:
        {
            return @"位置蓄積サーバ";
        }
        default:
            return nil;
    }
}

/* セルのタイトルを返す */
- (NSString *)_getCellTitle:(NSInteger)row inSection:(NSInteger)section
{
    switch (section)
    {
        case kSectionTop:
        {
            switch (row)
            {
                case kSectionTopCellTransmitToServer:
                {
                    return @"位置をサーバに送信する";
                }
                case kSectionTopCellLogUploadCount:
                {
                    return @"位置蓄積件数";
                }
                case kSectionTopCellDebugMode:
                {
                    return @"デバッグモード";
                }
                case kSectionTopCellDeleteLogFile:
                {
                    return @"ログファイル削除";
                }
                default:
                    return nil;
            }
        }
        case kSectionTokenServer:
        {
            switch (row)
            {
                case kSectionTokenServerCellClientID:
                {
                    return @"クライアントID";
                }
                case kSectionTokenServerCellSecret:
                {
                    return @"秘密鍵";
                }
                case kSectionTokenServerCellUrl:
                {
                    return @"URL";
                }
                default:
                    return nil;
            }
        }
        case kSectionLocationServer:
        {
            switch (row)
            {
                case kSectionLocationServerCellApiKey:
                {
                    return @"APIキー";
                }
                case kSectionLocationServerCellUrl:
                {
                    return @"URL";
                }
                case kSectionLocationServerCellDeviceID:
                {
                    return @"端末ID";
                }
                case kSectionLocationServerCellExtra:
                {
                    return @"付加情報";
                }
                case kSectionLocationServerCellUnsearchableExtra:
                {
                    return @"非検索付加情報";
                }
                case kSectionLocationServerCellIsSearch:
                {
                    return @"検索DB可能フラグ";
                }
                default:
                    return nil;
            }
        default:
            return nil;
        }
    }
}
@end
