//
//  LibSettingViewController.m
//  LocationApp_Sample
//
//  Copyright (c) 2015 ZENRIN DataCom CO., LTD. All Rights Reserved.
//

#import "LibSettingViewController.h"

#import "SettingCell.h"

@interface LibSettingViewController ()
{
    /* TableViewの情報リスト */
    __strong NSArray *_tableViewList;
    /* TableViewの情報リスト */
    UITableViewCell *_pullDownCell;
}

/* セクション定義 */
enum
{
    kSectionLibraryCertification = 0,
    kSectionMeasuringSettings,
};

/* "ライブラリ認証用"セクション定義 */
enum
{
    kSectionLibraryCertificationCellApiKey = 0,
    kSectionLibraryCertificationCellClientID,
    kSectionLibraryCertificationCellSecret,
    kSectionLibraryCertificationCellAuthEnv,

    kLibSettingSectionLibraryCertificationCellCount
};

/* "測位設定"セクションのセル定義 */
enum
{
    kSectionMeasuringSettingsCellAccuracy = 0,
    kSectionMeasuringSettingsCellDistance,
    kSectionMeasuringSettingsCellBatteryLevInterval,
    kSectionMeasuringSettingsCellSignificantChangeAccuracy,
    kSectionMeasuringSettingsCellLowEnergyMode,
    kSectionMeasuringSettingsCellMonitoringAreaSmallRadius,
    kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius,
    kSectionMeasuringSettingsCellMonitoringAreaLargeRadius,
    kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius,
    kSectionMeasuringSettingsCellMonitoringSlowSpeed,
    kSectionMeasuringSettingsCellMonitoringMiddleSpeed,
    kSectionMeasuringSettingsCellMonitoringHighSpeed,
    
    kSectionMeasuringSettingsCellCount
};

/* 設定値を更新 */
- (void)_textFieldValueChanged:(UITextField *)textfield;

/* データ内のsectionを返す */
- (NSInteger)_sectionOfData:(NSInteger)tableViewSection;
/* データ内のindexPathを返す */
- (NSIndexPath *)_indexPathOfData:(NSIndexPath *)tableViewIndexPath;

/* "ライブラリ認証用"セクションのセルを返す */
- (UITableViewCell *)_cellForLibraryCertificationSectionAtRow:(NSInteger)row;
/* "測位設定"セクションのセルを返す */
- (UITableViewCell *)_cellForMeasuringSettingsSectionAtRow:(NSInteger)row;

@end


@implementation LibSettingViewController

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder:aDecoder];
    if (self != nil)
    {
        _tableViewList = nil;
    }
    return self;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    if (_isEditable == NO) {
        [self.resetSettingButton setEnabled:NO];
        self.resetSettingButton.tintColor = [UIColor colorWithWhite:0 alpha:0];
    }
    
    [self _getNavigationTitle];
    [self _setupTableViewList];
    [self.tableView reloadData];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    if (_isEditable) {
        /* ライブラリ設定値を更新 */
        [self.dataManager saveLibSettings];
    }
}

#pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return _tableViewList.count;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [self _getSectionTitle:section];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger numberOfRows = 0;
    
    if (section < _tableViewList.count)
    {
        NSDictionary *sectionInfo = _tableViewList[section];
        numberOfRows = [sectionInfo[kCellsKey] count];
    }
    return numberOfRows;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    indexPath = [self _indexPathOfData:indexPath];
    
    switch (indexPath.section)
    {
        case kSectionLibraryCertification:
        {
            if (_isEditable) {
                return [self _cellForLibraryCertificationSectionAtRow:indexPath.row];
            } else {
                return [self _cellForFixedLibraryCertificationSectionAtRow:indexPath.row];
            }
        }
        case kSectionMeasuringSettings:
        {
            if (_isEditable) {
                return [self _cellForMeasuringSettingsSectionAtRow:indexPath.row];
            } else {
                return [self _cellForFixedMeasuringSettingsSectionAtRow:indexPath.row];
            }
        }
        default:
            return nil;
    }
}

#pragma mark UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    indexPath = [self _indexPathOfData:indexPath];
    
    SettingCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    
    switch (indexPath.section)
    {
        case kSectionLibraryCertification:
        {
            switch (indexPath.row)
            {
                case kSectionLibraryCertificationCellApiKey:
                case kSectionLibraryCertificationCellClientID:
                case kSectionLibraryCertificationCellSecret:
                {
                    if (_isEditable)
                    {
                        [self _showInputAlert:@"ライブラリ認証用" cell:cell];
                    }
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionMeasuringSettings:
        {
            switch (indexPath.row)
            {
                case kSectionMeasuringSettingsCellAccuracy:
                case kSectionMeasuringSettingsCellDistance:
                case kSectionMeasuringSettingsCellBatteryLevInterval:
                case kSectionMeasuringSettingsCellSignificantChangeAccuracy:
                case kSectionMeasuringSettingsCellMonitoringAreaSmallRadius:
                case kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius:
                case kSectionMeasuringSettingsCellMonitoringAreaLargeRadius:
                case kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius:
                case kSectionMeasuringSettingsCellMonitoringSlowSpeed:
                case kSectionMeasuringSettingsCellMonitoringMiddleSpeed:
                case kSectionMeasuringSettingsCellMonitoringHighSpeed:
                {
                    if (_isEditable) {
                        [self _showInputAlert:@"測位設定" cell:cell];
                    }
                    break;
                }
                default:
                    break;
            }
        }
    }
}

#pragma mark IBAction

- (IBAction)resetSetting:(id)sender
{
    [self showResetAlert:^{
        [self.dataManager resetLibSettings];
        [self.tableView reloadData];
    }];
}

- (IBAction)switchValueChanged:(id)sender
{
    NSIndexPath *indexPath = [self indexPathForAccessoryView:sender];
    if (indexPath == nil)
    {
        return;
    }
        
    switch (indexPath.section)
    {
        case kSectionMeasuringSettings:
        {
            switch (indexPath.row)
            {
                case kSectionMeasuringSettingsCellLowEnergyMode:
                {
                    self.dataManager.lowEnergyMode = [sender isOn];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }
}

#pragma mark Private

/* 設定値を更新 */
- (void)_textFieldValueChanged:(UITextField *)textfield
{
    NSIndexPath *indexPath = [self indexPathForAccessoryView:textfield];
    if (indexPath == nil)
    {
        return;
    }
    
    NSString *text = ( textfield.text.length == 0 )? nil: textfield.text;
    
    switch (indexPath.section)
    {
        case kSectionLibraryCertification:
        {
            switch (indexPath.row)
            {
                case kSectionLibraryCertificationCellApiKey:
                {
                    self.dataManager.libApiKey = text;
                    break;
                }
                case kSectionLibraryCertificationCellClientID:
                {
                    self.dataManager.libTokenClientID = text;
                    break;
                }
                case kSectionLibraryCertificationCellSecret:
                {
                    self.dataManager.libTokenSecret = text;
                    break;
                }
                default:
                    break;
            }
            break;
        }
        case kSectionMeasuringSettings:
        {
            switch (indexPath.row)
            {
                case kSectionMeasuringSettingsCellAccuracy:
                {
                    self.dataManager.accuracy = (text == nil)? FloatEmptyValue: [text floatValue];
                    textfield.text = [self textFromFloat:self.dataManager.accuracy];
                    break;
                }
                case kSectionMeasuringSettingsCellDistance:
                {
                    self.dataManager.distance = (text == nil)? FloatEmptyValue: [text floatValue];
                    textfield.text = [self textFromFloat:self.dataManager.distance];
                    break;
                }
                case kSectionMeasuringSettingsCellBatteryLevInterval:
                {
                    self.dataManager.batteryLevInterval = (text == nil)? FloatEmptyValue: [text floatValue];
                    textfield.text = [self textFromFloat:self.dataManager.batteryLevInterval];
                    break;
                }
                case kSectionMeasuringSettingsCellSignificantChangeAccuracy:
                {
                    self.dataManager.significantChangeAccuracy = (text == nil)? FloatEmptyValue: [text floatValue];
                    textfield.text = [self textFromFloat:self.dataManager.significantChangeAccuracy];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringAreaSmallRadius:
                {
                    self.dataManager.monitoringAreaSmallRadius = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringAreaSmallRadius];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius:
                {
                    self.dataManager.monitoringAreaMiddleRadius = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringAreaMiddleRadius];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringAreaLargeRadius:
                {
                    self.dataManager.monitoringAreaLargeRadius = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringAreaLargeRadius];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius:
                {
                    self.dataManager.monitoringAreaOversizeRadius = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringAreaOversizeRadius];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringSlowSpeed:
                {
                    self.dataManager.monitoringSlowSpeed = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringSlowSpeed];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringMiddleSpeed:
                {
                    self.dataManager.monitoringMiddleSpeed = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringMiddleSpeed];
                    break;
                }
                case kSectionMeasuringSettingsCellMonitoringHighSpeed:
                {
                    self.dataManager.monitoringHighSpeed = (text == nil)? DoubleEmptyValue: [text doubleValue];
                    textfield.text = [self textFromDouble:self.dataManager.monitoringHighSpeed];
                    break;
                }
                default:
                    break;
            }
            break;
        }
        default:
            break;
    }
}

/* 表示リストのセットアップ */
- (void)_setupTableViewList
{
    if (self.dataManager == nil)
    {
        return;
    }
    
    NSMutableArray *tableViewList = [NSMutableArray new];
    
    /* "ライブラリ認証用"セクション */
    NSMutableArray *locationServerCells = [NSMutableArray arrayWithCapacity:kSectionLibraryCertification];
    if (_isEditable || self.dataManager.libApiKey != nil)
    {
        [locationServerCells addObject:@(kSectionLibraryCertificationCellApiKey)];
    }
    if (_isEditable || self.dataManager.libTokenClientID != nil)
    {
        [locationServerCells addObject:@(kSectionLibraryCertificationCellClientID)];
    }
    if (_isEditable || self.dataManager.libTokenSecret != nil)
    {
        [locationServerCells addObject:@(kSectionLibraryCertificationCellSecret)];
    }
    if (_isEditable || self.dataManager.libAuthEnv != IntEmptyValue)
    {
        [locationServerCells addObject:@(kSectionLibraryCertificationCellAuthEnv)];
    }
    [tableViewList addObject:@{kSectionKey: @(kSectionLibraryCertification), kCellsKey: locationServerCells}];
    
    /* "測位設定"セクション */
    NSMutableArray *measuringSettingsCells = [NSMutableArray arrayWithCapacity:kSectionMeasuringSettingsCellCount];
    if (_isEditable || self.dataManager.accuracy != FloatEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellAccuracy)];
    }
    if (_isEditable || self.dataManager.distance != FloatEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellDistance)];
    }
    if (_isEditable || self.dataManager.batteryLevInterval != FloatEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellBatteryLevInterval)];
    }
    if (_isEditable || self.dataManager.significantChangeAccuracy != FloatEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellSignificantChangeAccuracy)];
    }
    [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellLowEnergyMode)];
    if (_isEditable || self.dataManager.monitoringAreaSmallRadius != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringAreaSmallRadius)];
    }
    if (_isEditable || self.dataManager.monitoringAreaMiddleRadius != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius)];
    }
    if (_isEditable || self.dataManager.monitoringAreaLargeRadius != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringAreaLargeRadius)];
    }
    if (_isEditable || self.dataManager.monitoringAreaOversizeRadius != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius)];
    }
    if (_isEditable || self.dataManager.monitoringSlowSpeed != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringSlowSpeed)];
    }
    if (_isEditable || self.dataManager.monitoringMiddleSpeed != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringMiddleSpeed)];
    }
    if (_isEditable || self.dataManager.monitoringHighSpeed != DoubleEmptyValue)
    {
        [measuringSettingsCells addObject:@(kSectionMeasuringSettingsCellMonitoringHighSpeed)];
    }
    
    [tableViewList addObject:@{kSectionKey: @(kSectionMeasuringSettings), kCellsKey: measuringSettingsCells}];

    _tableViewList = tableViewList;
}

/* データ内のsectionを返す */
- (NSInteger)_sectionOfData:(NSInteger)tableViewSection
{
    NSInteger section = NSNotFound;
    
    if (tableViewSection < _tableViewList.count)
    {
        NSDictionary *sectionInfo = _tableViewList[tableViewSection];
        section = [sectionInfo[kSectionKey] integerValue];
    }
    return section;
}

/* データ内のindexPathを返す */
- (NSIndexPath *)_indexPathOfData:(NSIndexPath *)tableViewIndexPath
{
    NSInteger section = NSNotFound;
    NSInteger row = NSNotFound;
    
    if (tableViewIndexPath.section < _tableViewList.count)
    {
        NSDictionary *sectionInfo = _tableViewList[tableViewIndexPath.section];
        NSArray *cells = sectionInfo[kCellsKey];
        
        if (tableViewIndexPath.row < cells.count)
        {
            section = [sectionInfo[kSectionKey] integerValue];
            row = [cells[tableViewIndexPath.row] integerValue];
        }
    }
    return [NSIndexPath indexPathForRow:row inSection:section];
}

/* "ライブラリ認証用"（変更可能）セクションのセルを返す */
- (UITableViewCell *)_cellForLibraryCertificationSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionLibraryCertification];
    
    switch (row)
    {
        case kSectionLibraryCertificationCellApiKey:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.libApiKey keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLibraryCertificationCellClientID:
        {
            return [self longTitleTextFieldCell:title text:self.dataManager.libTokenClientID keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLibraryCertificationCellSecret:
        {
            return [self shortTitleTextFieldCell:title text:self.dataManager.libTokenSecret keyboardType:UIKeyboardTypeASCIICapable];
        }
        case kSectionLibraryCertificationCellAuthEnv:
        {
            _pullDownCell = [self pullDownCell:title menu:[self _setupAuthDevMenu:row] selectedMenu:[self _getAuthDevTitle:self.dataManager.libAuthEnv]];
            return _pullDownCell;
        }
        default:
            return nil;
    }
}

/* "測位設定"（変更可能）セクションのセルを返す */
- (UITableViewCell *)_cellForMeasuringSettingsSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionMeasuringSettings];

    switch (row)
    {
        case kSectionMeasuringSettingsCellAccuracy:
        {
            return [self longTitleTextFieldCell:title text:[self textFromFloat:self.dataManager.accuracy] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellDistance:
        {
            return [self longTitleTextFieldCell:title text:[self textFromFloat:self.dataManager.distance] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellBatteryLevInterval:
        {
            return [self longTitleTextFieldCell:title text:[self textFromFloat:self.dataManager.batteryLevInterval] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellSignificantChangeAccuracy:
        {
            return[self longTitleTextFieldCell:title text:[self textFromFloat:self.dataManager.significantChangeAccuracy] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellLowEnergyMode:
        {
            return [self switchCell:title isOn:self.dataManager.lowEnergyMode];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaSmallRadius:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaSmallRadius] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaMiddleRadius] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaLargeRadius:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaLargeRadius] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaOversizeRadius] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringSlowSpeed:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringSlowSpeed] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringMiddleSpeed:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringMiddleSpeed] keyboardType:UIKeyboardTypeNumberPad];
        }
        case kSectionMeasuringSettingsCellMonitoringHighSpeed:
        {
            return [self longTitleTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringHighSpeed] keyboardType:UIKeyboardTypeNumberPad];
        }
        default:
            return nil;
    }
}

/* "ライブラリ認証用"（変更不可）セクションのセルを返す */
- (UITableViewCell *)_cellForFixedLibraryCertificationSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionLibraryCertification];
    
    switch (row)
    {
        case kSectionLibraryCertificationCellApiKey:
        {
            return [self shortTitleFixedTextFieldCell:title text:self.dataManager.libApiKey];
        }
        case kSectionLibraryCertificationCellClientID:
        {
            return [self longTitleFixedTextFieldCell:title text:self.dataManager.libTokenClientID];
        }
        case kSectionLibraryCertificationCellSecret:
        {
            return [self shortTitleFixedTextFieldCell:title text:self.dataManager.libTokenSecret];
        }
        case kSectionLibraryCertificationCellAuthEnv:
        {
            return [self longTitleFixedTextFieldCell:title text:[self _getAuthDevTitle:self.dataManager.libAuthEnv]];   
        }
        default:
            return nil;
    }
}

/* "測位設定"（変更不可）セクションのセルを返す */
- (UITableViewCell *)_cellForFixedMeasuringSettingsSectionAtRow:(NSInteger)row
{
    NSString *title = [self _getCellTitle:row inSection:kSectionMeasuringSettings];
    
    switch (row)
    {
        case kSectionMeasuringSettingsCellAccuracy:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromFloat:self.dataManager.accuracy]];
        }
        case kSectionMeasuringSettingsCellDistance:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromFloat:self.dataManager.distance]];
        }
        case kSectionMeasuringSettingsCellBatteryLevInterval:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromFloat:self.dataManager.batteryLevInterval]];
        }
        case kSectionMeasuringSettingsCellSignificantChangeAccuracy:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromFloat:self.dataManager.significantChangeAccuracy]];
        }
        case kSectionMeasuringSettingsCellLowEnergyMode:
        {
            return [self longTitleFixedTextFieldCell:title text:(self.dataManager.lowEnergyMode? @"オン": @"オフ")];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaSmallRadius:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaSmallRadius]];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaMiddleRadius]];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaLargeRadius:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaLargeRadius]];
        }
        case kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringAreaOversizeRadius]];
        }
        case kSectionMeasuringSettingsCellMonitoringSlowSpeed:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringSlowSpeed]];
        }
        case kSectionMeasuringSettingsCellMonitoringMiddleSpeed:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringMiddleSpeed]];
        }
        case kSectionMeasuringSettingsCellMonitoringHighSpeed:
        {
            return [self longTitleFixedTextFieldCell:title text:[self textFromDouble:self.dataManager.monitoringHighSpeed]];
        }
        default:
            return nil;
    }
}

/* アラート表示 */
- (void)_showInputAlert:(NSString *)title cell:(SettingCell *)cell
{
    [self showInputAlert:title cell:cell ok:^(NSString *value) {
       UITextField *_targetTextField = [cell textFieldInCell];
        _targetTextField.text = value;
        [self _textFieldValueChanged:_targetTextField];
    }];
}

/* NavigationBarのタイトルを返す */
- (void)_getNavigationTitle
{
    if (self.isEditable)
    {
        self.navigationItem.title = @"ライブラリ設定";
    }
    else
    {
        self.navigationItem.title = @"設定値取得";
    }
}

/* セクションのタイトルを返す */
- (NSString *)_getSectionTitle:(NSInteger)section
{
    switch (section)
    {
        case kSectionLibraryCertification:
        {
            return @"ライブラリ認証用";
        }
        case kSectionMeasuringSettings:
        {
            return @"測位設定";
        }
        default:
            return nil;
    }
}

/* セルのタイトルを返す */
- (NSString *)_getCellTitle:(NSInteger)row inSection:(NSInteger)section
{
    switch (section)
    {
        case kSectionLibraryCertification:
        {
            switch (row)
            {
                case kSectionLibraryCertificationCellApiKey:
                {
                    return @"APIキー";
                }
                case kSectionLibraryCertificationCellClientID:
                {
                    return @"クライアントID";
                }
                case kSectionLibraryCertificationCellSecret:
                {
                    return @"秘密鍵";
                }
                case kSectionLibraryCertificationCellAuthEnv:
                {
                    return @"認証サーバ環境";
                }
                default:
                    return nil;
            }
        }
        case kSectionMeasuringSettings:
        {
            switch (row)
            {
                case kSectionMeasuringSettingsCellAccuracy:
                {
                    return @"測位精度(m)";
                }
                case kSectionMeasuringSettingsCellDistance:
                {
                    return @"距離(m)";
                }
                case kSectionMeasuringSettingsCellBatteryLevInterval:
                {
                    return @"バッテリー残量";
                }
                case kSectionMeasuringSettingsCellSignificantChangeAccuracy:
                {
                    return@"測位切替精度";
                }
                case kSectionMeasuringSettingsCellLowEnergyMode:
                {
                    return @"省電力モード";
                }
                case kSectionMeasuringSettingsCellMonitoringAreaSmallRadius:
                {
                    return @"監視領域半径(小)(m)";
                }
                case kSectionMeasuringSettingsCellMonitoringAreaMiddleRadius:
                {
                    return @"監視領域半径(中)(m)";
                }
                case kSectionMeasuringSettingsCellMonitoringAreaLargeRadius:
                {
                    return @"監視領域半径(大)(m)";
                }
                case kSectionMeasuringSettingsCellMonitoringAreaOversizeRadius:
                {
                    return @"監視領域半径(特大)(m)";
                }
                case kSectionMeasuringSettingsCellMonitoringSlowSpeed:
                {
                    return @"監視移動速度(低)(km/h)";
                }
                case kSectionMeasuringSettingsCellMonitoringMiddleSpeed:
                {
                    return @"監視移動速度(中)(km/h)";
                }
                case kSectionMeasuringSettingsCellMonitoringHighSpeed:
                {
                    return @"監視移動速度(高)(km/h)";
                }
                default:
                    return nil;
            }
        default:
            return nil;
        }
    }
}

/* NavigationBarのタイトルを返す */
- (NSString *)_getAuthDevTitle:(NSInteger)menuId
{
    switch (menuId)
    {
        case AuthEnv_Test:
        {
            return @"内部検証";
        }
        case AuthEnv_Stg:
        {
            return @"顧客検証";
        }
        case AuthEnv_Pro:
        {
            return @"顧客本番";
        }
        default:
            return nil;
    }
}

/* 認証サーバ環境用PullDownメニュー定義 */
- (UIMenu *)_setupAuthDevMenu:(NSInteger)row
{
    /* メニュー選択後Block */
    void (^selected)(int) = ^(int authEnv) {
        self.dataManager.libAuthEnv = authEnv;
        SettingCell *cell = (SettingCell*)self->_pullDownCell;
        UIButton *pullDown = [cell pullDownCell];
        [pullDown setTitle:[self _getAuthDevTitle:authEnv] forState:UIControlStateNormal];
    };
    /* メニュー選択時コールバックアクション設定 */
    NSMutableArray* actions = [[NSMutableArray alloc] init];
    [actions addObject:[UIAction actionWithTitle:[self _getAuthDevTitle:AuthEnv_Test] image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Test);
    }]];
    [actions addObject:[UIAction actionWithTitle:[self _getAuthDevTitle:AuthEnv_Stg] image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Stg);
    }]];
    [actions addObject:[UIAction actionWithTitle:[self _getAuthDevTitle:AuthEnv_Pro] image:nil identifier:nil handler:^(__kindof UIAction* _Nonnull action) {
        selected(AuthEnv_Pro);
    }]];
    return [UIMenu menuWithTitle:@"" children:actions];
}

@end
